import sys
sys.path.append('./coin-press')
import numpy as np
import math
from avg_algos import FC_Avg,FC_Avg_UnknownDiam
import argparse

def L2(est):
    return np.linalg.norm(est)

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--rho', default=0.5, type=float, help='Total privacy budget')
    parser.add_argument('--d', default=50, type=int, help='Feature dimension (dimension of synthetic data)')
    parser.add_argument('--n', default=1000, type=int, help='Number of samples to synthesize (for synthetic data)')
    
    opt = parser.parse_args()
    return opt


def demo_mean(args):
    d = args.d
    n = args.n
    rho = args.rho
    mean = [0.0]*d
    cov = np.eye(d)
    beta = 0.01
    delta = 1/(n**2)
    R = 10*math.sqrt(d)
    
    X = np.random.multivariate_normal(mean, cov, int(n))
    non_private_mean = L2(np.mean(X, axis=0)-mean)
    print('non_private_mean {}\n'.format(non_private_mean))   
    
    
    print('Calling FC_Avg_KnownDiam ....')
    ta = FC_Avg(X,d,rho,delta)
    if ta is None:
        print("FriendlyCore has failed")
        return
    
    friendly_core = L2(np.asarray(ta))
    
    
    print('*******  FC_Avg_KnownDiam distance from average: {}  *******\n\n'.format(math.fabs(non_private_mean - friendly_core)))
    
    
    
    print('Calling FC_Avg_UnknownDiam ....')
    
    ha = FC_Avg_UnknownDiam(X,d,rho,delta,beta,r_min=0.01,r_max=10*math.sqrt(d))
    if ha is None:
        print("FC_Avg_UnknownDiam has failed")
        return
    
    friendly_core = L2(np.asarray(ha))
    print('******* FC_Avg_UnknownDiam distance from average: {} *******\n\n'.format(math.fabs(non_private_mean - friendly_core)))
    
    

if __name__ == '__main__':
    args = parse_args()

    print('~~ Begin mean estimation demo ~~') 
    demo_mean(args)
