from clustering_algos import FC_kTupleClustering
import numpy as np
import math
import random
from time import sleep
from avg_algos import l2_dist_squared
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
from clustering_algos import Points,FC_Clustering
from clustering_utils import non_private_kMeans_Clustering_0,non_private_kMeans_Clustering_1
    

'''
GenSamples: Generating synthetic database for clustering demo (d=2, k=3).

Inputs:
n: Number of samples.
x: Separation parameter.
'''
def GenSamples(n, x):
    y = math.ceil(math.sqrt(3)*x)
    Means = [[x,0],[-x,0],[0,y]]
    Covs = np.array([[[1,0],[0,0.7]],[[0.2,0],[0,1.5]],[[0.3,0.2],[0.2,1.5]]])
    S = []
    ofrac = 0.1
    gfrac = (1-ofrac)/3
    Parameters = (n,300*gfrac,Means[0][0],Means[0][1],Means[1][0],Means[1][1],Means[2][0],Means[2][1],100*ofrac)
    print("Generating %d samples, %d%% from 3 small variance Gaussians centered at ([%lf,%lf], [%lf,%lf],[%lf,%lf]), and %d%% from 3 high variance Gaussians (same centers)" % Parameters)
    sleep(4)
    for _ in range(n):
        i = np.random.randint(3)
        x = np.random.multivariate_normal(Means[i],80*Covs[i])
        rand = random.random()
        if rand < gfrac:
            x = np.random.multivariate_normal(Means[0],Covs[0])
        elif gfrac <= rand < 2*gfrac:
            x = np.random.multivariate_normal(Means[1],Covs[1])
        elif 2*gfrac <= rand < 3*gfrac:
            x = np.random.multivariate_normal(Means[2],Covs[2])
        S.append(x)
    return np.array(S)



if __name__ == '__main__':
    
    
    print("********  Clustering a small database with large separation *************")
    S = GenSamples(n=25000, x=6)
    Centers = FC_Clustering(S,d=2,k=3,rho=1,delta=1e-8,beta=0.05,r_min=0.01,Lambda=50,num_of_tuples=300,non_private_alg=non_private_kMeans_Clustering_0,use_Lloyd=True)
    print("The resulting private centers:")
    print(Centers)
    
    print("\n\n********  Attempt to cluster a small database with small separation *************")
    try:
        S = GenSamples(n=25000, x=2.5)
        Centers = FC_Clustering(S,d=2,k=3,rho=1,delta=1e-8,beta=0.05,r_min=0.01,Lambda=50,num_of_tuples=300,non_private_alg=non_private_kMeans_Clustering_0,use_Lloyd=True)
        print("The resulting private centers:")
        print(Centers)
    except:
        print("FriendlyCore Clustering has failed (as expected).") 
    

    print("\n\n********  Clustering a large database with small separation *************")
    S = GenSamples(n=150000, x=2.5)
    Centers = FC_Clustering(S,d=2,k=3,rho=1,delta=1e-8,beta=0.05,r_min=0.01,Lambda=50,num_of_tuples=300,non_private_alg=non_private_kMeans_Clustering_0,use_Lloyd=True)
    print("The resulting private centers:")
    print(Centers)

    
 

