from sklearn.cluster import KMeans,MiniBatchKMeans
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
import numpy as np
from dist_utils import l2_dist_squared
import itertools
from statistics import mode


def compute_kmeans_cost(data, centers): 
    cost = 0
    for x in data:
        cost = cost + np.min([l2_dist_squared(x, c) for c in centers])
    return cost
 

def compute_success_rate(data,labels,centers):
    n = len(data)
    k = len(centers)
    new_labels = []
    for i in range(n):
        j = np.argmin([l2_dist_squared(data[i], c) for c in centers])
        new_labels.append(j)     
       
    min_diffs = n
    for pi in itertools.permutations(range(k)):
        diffs = 0
        for i in range(n):
            if new_labels[i]!=pi[labels[i]]:
                diffs = diffs + 1 
        min_diffs = min(diffs,min_diffs)    
    return 1-min_diffs/n
    

def Lloyd_step(data,centers):
    k = len(centers)
    Clusters = []
    for _ in range(k):
        Clusters = Clusters + [[]]
    for x in data:
        i = np.argmin([l2_dist_squared(x, c) for c in centers])
        Clusters[i].append(x)
    for i in range(k):
        cluster_i = np.array(Clusters[i])
        centers[i] = np.average(cluster_i, axis=0)
    return centers


def non_private_kMeans_Clustering(D,k,lloyd_steps):
    kmeans = KMeans(n_clusters=k, random_state=0).fit(D)
    centers = kmeans.cluster_centers_
    for _ in range(lloyd_steps):
        centers = Lloyd_step(D,centers)
        
    return centers

def non_private_kMeans_Clustering_10(D,k):
    return non_private_kMeans_Clustering(D,k,10)

def non_private_kMeans_Clustering_5(D,k):
    return non_private_kMeans_Clustering(D,k,5)

def non_private_kMeans_Clustering_4(D,k):
    return non_private_kMeans_Clustering(D,k,1)

def non_private_kMeans_Clustering_3(D,k):
    return non_private_kMeans_Clustering(D,k,3)

def non_private_kMeans_Clustering_2(D,k):
    return non_private_kMeans_Clustering(D,k,2)

def non_private_kMeans_Clustering_1(D,k):
    return non_private_kMeans_Clustering(D,k,1)

def non_private_kMeans_Clustering_0(D,k):
    return non_private_kMeans_Clustering(D,k,0)


def non_private_PCA_Clusters(D,k):
    n = len(D)
    scaler = StandardScaler()
    scaler.fit(D)
    X = scaler.transform(D) 
    
    #pca = PCA(.9)  
    pca = PCA(n_components=k)
    #pca = PCA(n_components=2)    
    pca.fit(X)
    X = pca.transform(X)
    kmeans = KMeans(n_clusters=k, random_state=0).fit(X)
    centers = kmeans.cluster_centers_
    
    Clusters = []
    for _ in range(k):
        Clusters = Clusters + [[]]
    for i in range(n):
        j = np.argmin([l2_dist_squared(X[i], c) for c in centers])
        Clusters[j].append(D[i])
        
    return Clusters
        
def non_private_PCA_Clustering(D,k):
    centers = []
    Clusters = non_private_PCA_Clusters(D,k)
    for i in range(k):
        cluster_i = np.array(Clusters[i])
        centers.append(np.average(cluster_i, axis=0))
        
    return centers

