from clustering_algos import FC_Clustering
from clustering import clustering_algorithm
from clustering import clustering_params
from clustering_utils import non_private_kMeans_Clustering_3
import numpy as np


'''
A wrapper for the clustering method of Google that is based on Locality Sensitive Hashing (LSH).
'''
def LSH_Clustering(D,d,k,eps,delta,radius):
    privacy_param = clustering_params.DifferentialPrivacyParam(epsilon=eps, delta=delta)
    lsh_data = clustering_params.Data(np.array(D), radius, labels=None)
    lsh_results = clustering_algorithm.private_lsh_clustering(k, lsh_data, privacy_param)
    lsh_centers = lsh_results.centers
    return lsh_centers

'''
A wrapper for FriendlyCore Clustering, that uses a tiny privacy budget for executing LSH_Clustering as backup (in case of a failure).
'''
def FC_Clustering_Wrapper(D,d,k,rho,delta,beta,r_min,Lambda,num_of_tuples,non_private_alg=non_private_kMeans_Clustering_3):
    fc_centers = None
    try:
        fc_centers = FC_Clustering(D,d,k,0.99*rho,0.99*delta,beta,r_min,Lambda,num_of_tuples,non_private_alg=non_private_alg,use_Lloyd=True)
    except:
        print("**** FriendlyCore has failed! Executing LSH as backup ... *****")
        eps = (0.02*rho)**0.5
        fc_centers = LSH_Clustering(D,d,k,eps,0.01*delta,radius=Lambda)
    return fc_centers