import sys
sys.path.append('./coin-press')
import numpy as np
import math
from algos import multivariate_mean_iterative,L2
from scipy import stats
from avg_algos import FC_Avg


def Test_R(points=9,iterations=50):
    # Effect of varying r. Fix parameters n = 800, d = 1000, rho = 1. 
    d = 1000
    mean = [0.0]*d
    cov = np.eye(d)
    c = [0]*d
    n = 800
    rho = 1
    delta = 1e-8
    err_nonpr = []
    
    err_t4_coinpress = []
    err_t40_coinpress = []
    err_t50_coinpress = []
    err_t20_coinpress = []
    err_fc_avg = []
    counters = []

    Ps4_coinpress = [(1.0/12.0)*rho, (1.0/12.0)*rho, (1.0/12.0)*rho, (3.0/4.0)*rho]
    Ps50_coinpress = [rho/(4*49)]*49
    Ps50_coinpress.append((3.0/4.0)*rho)
    Ps20_coinpress = [rho/(4*19)]*19
    Ps20_coinpress.append((3.0/4.0)*rho)
    Ps40_coinpress = [rho/(4*39)]*39
    Ps40_coinpress.append((3.0/4.0)*rho)
    
    
    for i in np.linspace(1, 9, num=points):
        R = (10**i)*np.sqrt(d)

        non_pr = []
        means_t50_coinpress = []
        means_t20_coinpress = []
        means_t40_coinpress = []
        means_t4_coinpress = []
        means_fc_avg = []
        for j in range(iterations):
            print("log10(R) = %lf (iteration %d)" % (math.log10(R),j))
            X = np.random.multivariate_normal(mean, cov, int(n))
            non_pr.append(L2(np.mean(X, axis=0)-mean))
            
            means_t50_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, 50, Ps50_coinpress)-mean))
            means_t20_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, 20, Ps20_coinpress)-mean))
            means_t40_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, 40, Ps40_coinpress)-mean))
            means_t4_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, 4, Ps4_coinpress)-mean))
            ta = FC_Avg(X,d,rho,delta)
            means_fc_avg.append(L2(ta-mean))
        
        err_nonpr.append(stats.trim_mean(non_pr,0.1))
        err_t50_coinpress.append(stats.trim_mean(means_t50_coinpress,0.1))
        err_t20_coinpress.append(stats.trim_mean(means_t20_coinpress,0.1))
        err_t40_coinpress.append(stats.trim_mean(means_t40_coinpress,0.1))
        err_t4_coinpress.append(stats.trim_mean(means_t4_coinpress,0.1))
        err_fc_avg.append(stats.trim_mean(means_fc_avg,0.1))
        counters.append(math.log10(R))
        
        np.savetxt("./results/synthetic_mean/counters-R.txt", np.array(counters))
        np.savetxt("./results/synthetic_mean/nonpr-R.txt", np.array(err_nonpr))
        np.savetxt("./results/synthetic_mean/t50_coinpress-R.txt", np.array(err_t50_coinpress))
        np.savetxt("./results/synthetic_mean/t20_coinpress-R.txt", np.array(err_t20_coinpress))
        np.savetxt("./results/synthetic_mean/t40_coinpress-R.txt", np.array(err_t40_coinpress))
        np.savetxt("./results/synthetic_mean/t4_coinpress-R.txt", np.array(err_t4_coinpress))
        np.savetxt("./results/synthetic_mean/fc_avg-R.txt", np.array(err_fc_avg))
        
    print("Done")



if __name__ == '__main__':    
    Test_R()
