import sys
sys.path.append('./coin-press')
import numpy as np
import matplotlib.pyplot as plt
import math
from algos import multivariate_mean_iterative,L2
from scipy import stats
from avg_algos import FC_Avg

def Test_d(iterations=5):
    # Vary d. Fix parameters n = 800, R = 10*sqrt(d), rho = 1. 
    n = 800
    rho = 1
    delta = 1e-8
    
    err_nonpr = []
    err_t2_coinpress = []
    err_t4_coinpress = []
    err_t8_coinpress = []
    err_fc_avg = []
    counters = []
    
    rho_list_2 = [(1.0/2.0)*rho,(3.0/4.0)*rho]
    rho_list_4 = [(1.0/12.0)*rho, (1.0/12.0)*rho, (1.0/12.0)*rho, (3.0/4.0)*rho]
    rho_list_8 = [(1.0/28.0)*rho, (1.0/28.0)*rho, (1.0/28.0)*rho, (1.0/28.0)*rho, (1.0/28.0)*rho, (1.0/28.0)*rho, (1.0/28.0)*rho, (3.0/4.0)*rho]
    
    for d in [1,1000,2000,3000,4000,5000,6000,7000,8000,9000]:
        d = math.ceil(d)
        mean = [0.0]*d
        cov = np.eye(d)
        c = [0]*d
        R = 10*np.sqrt(d)
        
        non_pr = []
        means_t2_coinpress = []
        means_t4_coinpress = []
        means_t8_coinpress = []
        means_fc_avg = []
        for i in range(iterations):
            print("d = %d (iteration %d)" % (d,i))
            X = np.random.multivariate_normal(mean, cov, int(n))
            non_pr.append(L2(np.mean(X, axis=0)-mean))
            
            means_t2_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, len(rho_list_2), rho_list_2)-mean))         
            means_t4_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, len(rho_list_4), rho_list_4)-mean))
            means_t8_coinpress.append(L2(multivariate_mean_iterative(X.copy(), c, R, len(rho_list_8), rho_list_8)-mean))
            ta = FC_Avg(X,d,rho,delta)
            means_fc_avg.append(L2(ta-mean))
        
        err_nonpr.append(stats.trim_mean(non_pr,0.1))
        err_t2_coinpress.append(stats.trim_mean(means_t2_coinpress,0.1))
        err_t4_coinpress.append(stats.trim_mean(means_t4_coinpress,0.1))
        err_t8_coinpress.append(stats.trim_mean(means_t8_coinpress,0.1))
        err_fc_avg.append(stats.trim_mean(means_fc_avg,0.1))
        counters.append(d)
    
    np.savetxt("./results/synthetic_mean/counters-d.txt", np.array(counters))
    np.savetxt("./results/synthetic_mean/nonpr-d.txt", np.array(err_nonpr))
    np.savetxt("./results/synthetic_mean/t2_coinpress-d.txt", np.array(err_t2_coinpress))
    np.savetxt("./results/synthetic_mean/t4_coinpress-d.txt", np.array(err_t4_coinpress))
    np.savetxt("./results/synthetic_mean/t8_coinpress-d.txt", np.array(err_t8_coinpress))
    np.savetxt("./results/synthetic_mean/fc_avg-d.txt", np.array(err_fc_avg))
    
    plt.title('Multivariate Mean Estimation')
    
    print("Done")



if __name__ == '__main__':
    Test_d()

