import sys
sys.path.append('./coin-press')
import numpy as np
import matplotlib.pyplot as plt
import math
from algos import multivariate_mean_iterative,L2
from scipy import stats
from utils import fineMeanEst, privateRangeEst, twoShot
from avg_algos import FC_Avg
from time import sleep

def Test_n(points=8,tries=30):
    # Vary n. Fix parameters d = 50, R = 10*sqrt(d), rho = 1.
    d = 50
    mean = [0.0]*d
    cov = np.eye(d)
    c = [0]*d
    r = 10*np.sqrt(d)
    rho = 1
    eps = math.sqrt(2*rho)
    Ps2_coinpress = [(1.0/4.0)*rho, (3.0/4.0)*rho]
    Ps4_coinpress = [(1.0/12.0)*rho, (1.0/12.0)*rho, (1.0/12.0)*rho, (3.0/4.0)*rho]
    
    err_nonpr = []
    err_naive = []
    err_kv = []
    err_t2_coinpress = []
    err_t4_coinpress = []
    err_fc_avg = []
    counters = []
    
    for n in np.linspace(500, 5000, num=points):
        delta = 1e-8
        non_pr = []
        means_naive = []
        means_kv = []
        means_t2_coinpress = []
        means_t4_coinpress = []
        means_fc_avg = []
        for i in range(tries):
            print("n = %d (iteration %d)" % (n,i))
            X = np.random.multivariate_normal(mean, cov, int(n))
            real_avg = np.mean(X, axis=0)
            non_pr.append(L2(np.mean(X, axis=0)-mean))
            means_naive_coord = []
            means_kv_coord = []
            for j in range(d):
                means_naive_coord.append(fineMeanEst(X[:,j].copy(), 1, r, eps/np.sqrt(d)) - mean[j])
                means_kv_coord.append(twoShot(X[:,j].copy(), np.sqrt(1.0/2.0)*eps/np.sqrt(d), np.sqrt(1.0/2.0)*eps/np.sqrt(d), 0, r, 1) - mean[j])
            means_kv.append(L2(np.asarray(means_kv_coord)))
            tmp = L2(multivariate_mean_iterative(X.copy(), c, r, 2, Ps2_coinpress)-real_avg)
            means_t2_coinpress.append(tmp)
            print("******* CoinPress_t2 dist: %lf" % tmp)
            tmp = L2(multivariate_mean_iterative(X.copy(), c, r, 4, Ps4_coinpress)-real_avg)
            means_t4_coinpress.append(tmp)
            print("******* CoinPress_t4 dist: %lf" % tmp)
            ta = FC_Avg(X,d,rho,delta)
            print("******* FriendlyCore dist: %lf" % L2(ta-real_avg))
            means_fc_avg.append(L2(ta-real_avg))
            sleep(1)
        
        err_nonpr.append(stats.trim_mean(non_pr,0.1)) 
        err_naive.append(stats.trim_mean(means_naive,0.1))
        err_kv.append(stats.trim_mean(means_kv,0.1))
        err_t2_coinpress.append(stats.trim_mean(means_t2_coinpress,0.1))
        err_t4_coinpress.append(stats.trim_mean(means_t4_coinpress,0.1))
        err_fc_avg.append(stats.trim_mean(means_fc_avg,0.1))
        counters.append(n)
    
    np.savetxt("./results/synthetic_mean/counters-n.txt", np.array(counters))
    np.savetxt("./results/synthetic_mean/nonpr-n.txt", np.array(err_nonpr))
    np.savetxt("./results/synthetic_mean/naive-n.txt", np.array(err_naive))
    np.savetxt("./results/synthetic_mean/kv-n.txt", np.array(err_kv))
    np.savetxt("./results/synthetic_mean/t2_coinpress-n.txt", np.array(err_t2_coinpress))
    np.savetxt("./results/synthetic_mean/t4_coinpress-n.txt", np.array(err_t4_coinpress))
    np.savetxt("./results/synthetic_mean/fc_avg-n.txt", np.array(err_fc_avg))
        
    non_private = plt.scatter(x=counters,y=err_nonpr,s=2)
    naive = plt.scatter(x=counters,y=err_naive,s=2)
    kv = plt.scatter(x=counters,y=err_kv,s=2)
    t2_coinpress = plt.scatter(x=counters,y=err_t2_coinpress,s=2)
    t4_coinpress = plt.scatter(x=counters,y=err_t4_coinpress,s=2)
    fc_avg = plt.scatter(x=counters,y=err_fc_avg,s=2)
    plt.xlabel('n')
    plt.ylabel('Mean Est L2 Error')
    plt.legend(( non_private, naive, kv, t2_coinpress, t4_coinpress, fc_avg,),
               ('Non-private', 'Naive', 'CP-large-p', 'CP-small-p', 'EstAvg'),
               scatterpoints=1,
               loc='upper right',
               ncol=2,
               fontsize=12)
    plt.title('Multivariate Mean Estimation')
    
    print("Done")



if __name__ == '__main__':
    Test_n()
