import numpy as np
import math
import random
import time
import matplotlib.pyplot as plt
from clustering_utils import compute_success_rate,compute_kmeans_cost,non_private_PCA_Clustering,non_private_kMeans_Clustering_5,non_private_kMeans_Clustering_0
from private_clustering import FC_Clustering_Wrapper,LSH_Clustering
from clustering.demo import data_generation
from scipy import stats
import statistics

#from multiprocessing import Pool
#import concurrent.futures

__show_plots__ = 0


iterations = 30

#separation
cluster_ratio = 32
#num of tuples
#t = 100
t = 200
#samples per tuple
k = 8
rho = 1
eps=2
delta = 1e-8
beta = 0.05
radius=1


def GenSamples(n,d):
    data = data_generation.generate_synthetic_dataset(num_points=n,dim=d,num_clusters=k,cluster_ratio=cluster_ratio,radius=radius)
    print('==== Synthentic Dataset Information ====\n'
        f'Number of datapoints: {n}\n'
        f'Dimensions: {d}\n'
        f'Number of clusters: {k}\n'
        f'Radius: {radius}\n'
        f'Cluster centers drawn from: Uniform over ball of '
        f'radius={radius * (1 - 1 / float(cluster_ratio)):.4}\n'
        f'Each cluster drawn from: '
        f'N(cluster_center, '
        f'{radius / cluster_ratio / np.sqrt(d):.4} * I) '
        f'clipped to ball of radius {radius}')
    return (data.datapoints,data.labels)



def single_test(n,d, fc_loss_list,lsh_loss_list,fc_suc_list,lsh_suc_list):
    r_min = 0.001
    
    (samples,labels) = GenSamples(n,d)
    non_private_centers = non_private_kMeans_Clustering_0(samples,k)
    kmeans_cost = compute_kmeans_cost(samples,non_private_centers)
    fc_centers = FC_Clustering_Wrapper(samples,d,k,rho,delta,beta,r_min,Lambda = radius,num_of_tuples=t,non_private_alg = non_private_kMeans_Clustering_5)
    fc_success=-1
    fc_loss = 1-kmeans_cost/compute_kmeans_cost(samples,fc_centers)
    print("FriendlyCore k-means loss (1-OPT/COST) = %lf" % (fc_loss))   
    fc_suc_list.append(fc_success) 
    fc_loss_list.append(fc_loss)
    
    
       
    lsh_centers = LSH_Clustering(samples,d,k,eps,delta,radius)
    lsh_success=-1
    lsh_loss = 1-kmeans_cost/compute_kmeans_cost(samples,lsh_centers)
    print("LSH k-means loss (1-OPT/COST) = %lf" % (lsh_loss))  
    lsh_suc_list.append(lsh_success)
    lsh_loss_list.append(lsh_loss)
    if __show_plots__:
        fig, ax = plt.subplots()
        plt.scatter(samples[:,0],samples[:,1],s=1,color='black')
        plt.scatter(np.array(fc_centers)[:,0],np.array(fc_centers)[:,1],s=22,color='green')
        plt.scatter(np.array(lsh_centers)[:,0],np.array(lsh_centers)[:,1],s=10,color='red')
        filename = './plots/test_d2/FCvsLSH_2e5_%d.png' % len(fc_loss_list)
        plt.savefig(filename)
        plt.close(fig)
        #plt.show()
    
   
    
if __name__ == '__main__':
    
    d=2
    n_options = [10000,50000,100000,200000,300000,400000]
    num_of_n_options = len(n_options)
    
    fc_suc_lists = []
    fc_cost_lists = []
    lsh_cost_lists = []
    lsh_suc_lists = []
    for _ in range(num_of_n_options):
        fc_suc_lists = fc_suc_lists + [[]]
        fc_cost_lists = fc_cost_lists + [[]]
        lsh_cost_lists = lsh_cost_lists + [[]]
        lsh_suc_lists = lsh_suc_lists + [[]]
    
    fc_means = []
    fc_stdev = []
    fc_medians = []
    fc_perc90 = []
    fc_perc10 = []
    lsh_means = []
    lsh_stdev = []
    lsh_medians = []
    lsh_perc90 = []
    lsh_perc10 = []
    counters = []
        
    for i in range(num_of_n_options):
        n = n_options[i]
        counters.append(n)
        fc_loss_list = fc_cost_lists[i]
        lsh_loss_list = lsh_cost_lists[i]
        print("Testing n=%d" % n)
        for j in range(iterations):
            print("n=%d, iteration %d" % (n,j))
            single_test(n, d, fc_loss_list, lsh_loss_list, fc_suc_lists[i], lsh_suc_lists[i])
        print("********   n=%d Summary After Count %d  ********" % (n,i))
        print("FriendlyCore:")
        fc_means.append(stats.trim_mean(fc_loss_list,0.1)) 
        fc_stdev.append(statistics.pstdev(fc_loss_list))
        fc_medians.append(statistics.median(fc_loss_list))
        fc_perc90.append(np.percentile(fc_loss_list,90))
        fc_perc10.append(np.percentile(fc_loss_list,10))
        
        print(np.around(fc_means,decimals=3))
        print(np.around(fc_stdev,decimals=3))
        print(np.around(fc_medians,decimals=3))
        print(np.around(fc_perc90,decimals=3))
        print(np.around(fc_perc10,decimals=3))
        print("LSH:")
        lsh_means.append(stats.trim_mean(lsh_loss_list,0.1)) 
        lsh_stdev.append(statistics.pstdev(lsh_loss_list))
        lsh_medians.append(statistics.median(lsh_loss_list))
        lsh_perc90.append(np.percentile(lsh_loss_list,90))
        lsh_perc10.append(np.percentile(lsh_loss_list,10))
        print(np.around(lsh_means,decimals=3))
        print(np.around(lsh_stdev,decimals=3))
        print(np.around(lsh_medians,decimals=3))
        print(np.around(lsh_perc90,decimals=3))
        print(np.around(lsh_perc10,decimals=3))
        
        np.savetxt("./results/clustering/d2/counters.txt", np.array(counters))
        
        np.savetxt("./results/clustering/d2/fc_means.txt", np.array(fc_means))
        np.savetxt("./results/clustering/d2/fc_medians.txt", np.array(fc_medians))
        np.savetxt("./results/clustering/d2/fc_perc90.txt", np.array(fc_perc90))
        np.savetxt("./results/clustering/d2/fc_perc10.txt", np.array(fc_perc10))
        
        np.savetxt("./results/clustering/d2/lsh_means.txt", np.array(lsh_means))
        np.savetxt("./results/clustering/d2/lsh_medians.txt", np.array(lsh_medians))
        np.savetxt("./results/clustering/d2/lsh_perc90.txt", np.array(lsh_perc90))
        np.savetxt("./results/clustering/d2/lsh_perc10.txt", np.array(lsh_perc10))

    
    print("Done")
    
