import csv
from avg_algos import l2_dist_squared
from clustering_utils import compute_success_rate,compute_kmeans_cost,non_private_PCA_Clustering,non_private_kMeans_Clustering_0
from private_clustering import FC_Clustering_Wrapper,LSH_Clustering
import numpy as np
from time import sleep
import math
from scipy import stats
import statistics

from clustering import clustering_algorithm
from clustering import clustering_params

t = 200
beta = 0.05
rho = 1
eps = 2
delta = 1e-8
iterations = 20
r_min = 0.1
Lambda = 1e5


def add_gas_sensors(D,filename):
    f = open(filename, "r")
    line = f.readline()
    while line:
        line = f.readline()
        str_list = line.split()
        float_list = np.array([float(x) for x in str_list[3:]])
        if len(float_list)==16:
            D.append(float_list)
    f.close()


def single_test(S,k,fc_list,lsh_list):
    
    n = len(S)
    d = len(S[0])
    np.random.shuffle(S)
    D = S[:math.floor(0.9*n)]
    print("Number of points: %d" % len(D))

    print("Executing non private k-means")
    kmeans_centers = non_private_kMeans_Clustering_0(D,k)
    kmeans_cost = compute_kmeans_cost(D, kmeans_centers)
    print("sqrt(kmeans_cost) = %lf" % (kmeans_cost)**0.5)
    
    print("Executing FriendlyCore")
    #fc_centers = FC_Clustering_Wrapper(D,d,k,rho,delta,beta,r_min,r_max,num_of_tuples=t,non_private_alg = non_private_PCA_Clustering)
    fc_centers = FC_Clustering_Wrapper(D,d,k,rho,delta,beta,r_min,Lambda,num_of_tuples=t,non_private_alg = non_private_kMeans_Clustering_0)
    fc_loss = 1-kmeans_cost/compute_kmeans_cost(D,fc_centers)
    print("FriendlyCore k-means loss 1-OPT/COST = %.3f" % (fc_loss)) 
    fc_list.append(fc_loss)
    
    print("Executing LSH Clustering")   
    lsh_centers = LSH_Clustering(D,d,k,eps,delta,radius = Lambda)
    lsh_loss = 1-kmeans_cost/compute_kmeans_cost(D,lsh_centers)
    print("LSH k-means loss 1-OPT/COST = %.3f" % (lsh_loss))
    lsh_list.append(lsh_loss)
    
    
    
    
if __name__ == '__main__':    


    D = []
    add_gas_sensors(D,'ethylene_CO.txt')
    add_gas_sensors(D,'ethylene_methane.txt')
    
    k_options = [2,3,4,5,6]
    num_of_k_options = len(k_options)
    
    fc_lists = []
    lsh_lists = []
    for _ in range(num_of_k_options):
        fc_lists = fc_lists + [[]]
        lsh_lists = lsh_lists + [[]]
    
    fc_means = []
    fc_stdev = []
    fc_medians = []
    fc_perc90 = []
    fc_perc10 = []
    lsh_means = []
    lsh_stdev = []
    lsh_medians = []
    lsh_perc90 = []
    lsh_perc10 = []
    counters = []
    
    for i in range(num_of_k_options):
        k = k_options[i]
        counters.append(k)
        fc_list = fc_lists[i]
        lsh_list = lsh_lists[i]
        print("Testing k=%d" % k)
        for j in range(iterations):
            print("k=%d, iteration %d" % (k,j))
            single_test(D,k,fc_list,lsh_list)
        print("********   k=%d Summary After Count %d  ********" % (k,i))
        print("FriendlyCore:")
        fc_means.append(stats.trim_mean(fc_list,0.1)) 
        fc_stdev.append(statistics.pstdev(fc_list))
        fc_medians.append(statistics.median(fc_list))
        fc_perc90.append(np.percentile(fc_list,90))
        fc_perc10.append(np.percentile(fc_list,10))
        print(np.around(fc_means,decimals=3))
        print(np.around(fc_stdev,decimals=3))
        print(np.around(fc_medians,decimals=3))
        print(np.around(fc_perc90,decimals=3))
        print(np.around(fc_perc10,decimals=3))
        print("LSH:")
        lsh_means.append(stats.trim_mean(lsh_list,0.1)) 
        lsh_stdev.append(statistics.pstdev(lsh_list))
        lsh_medians.append(statistics.median(lsh_list))
        lsh_perc90.append(np.percentile(lsh_list,90))
        lsh_perc10.append(np.percentile(lsh_list,10))
        print(np.around(lsh_means,decimals=3))
        print(np.around(lsh_stdev,decimals=3))
        print(np.around(lsh_medians,decimals=3))
        print(np.around(lsh_perc90,decimals=3))
        print(np.around(lsh_perc10,decimals=3))
        
    np.savetxt("./results/clustering/gas/counters.txt", np.array(counters))
    
    np.savetxt("./results/clustering/gas/fc_means.txt", np.array(fc_means))
    np.savetxt("./results/clustering/gas/fc_medians.txt", np.array(fc_medians))
    np.savetxt("./results/clustering/gas/fc_perc90.txt", np.array(fc_perc90))
    np.savetxt("./results/clustering/gas/fc_perc10.txt", np.array(fc_perc10))
    
    np.savetxt("./results/clustering/gas/lsh_means.txt", np.array(lsh_means))
    np.savetxt("./results/clustering/gas/lsh_medians.txt", np.array(lsh_medians))
    np.savetxt("./results/clustering/gas/lsh_perc90.txt", np.array(lsh_perc90))
    np.savetxt("./results/clustering/gas/lsh_perc10.txt", np.array(lsh_perc10))
    
    print("Done")
    
    
    
    
