import numpy as np
import math
import random
import time
import matplotlib.pyplot as plt
from clustering_utils import compute_success_rate,non_private_PCA_Clustering,non_private_kMeans_Clustering_2
from private_clustering import FC_Clustering_Wrapper,LSH_Clustering
from scipy import stats
import statistics


iterations = 30


# number of clusters
k = 5

# num of tuples
t = 200

# samples per tuple
m = math.ceil(250*k)

# total num of samples
n = t*m

rho=1
delta = 1e-8
beta=0.05
eps=2



def GenSamples(n,d):
    print("dist between centers = %lf" % (math.sqrt(d/2)))
    print("dist from centers = %lf" % math.sqrt(d))
    
    Means = []
    for _ in range(k):
        Means = Means + [[]]
    for i in range(k):
        for _ in range(d):
            r = np.random.randint(2) + 1
            Means[i].append(r)
        
    
    samples = []
    labels = []
    for i in range(n):
        x = []
        if (i+1) % 10000 == 0:
            print("[GenSamples] iteration %d" % (i+1))
        i = np.random.randint(k)
        for j in range(d):
            x.append(Means[i][j]+np.random.randn())
        samples.append(x)
        labels.append(i)
    return (samples,labels)




def single_test(n,d,fc_list,lsh_list):
    

    (samples,labels) = GenSamples(n,d)
    r_min = 0.1
    Lambda=10*math.sqrt(d)
    
    non_private_method = non_private_PCA_Clustering
    fc_centers = FC_Clustering_Wrapper(samples,d,k,rho,delta,beta,r_min,Lambda,num_of_tuples=t,non_private_alg=non_private_method)
    fc_success = compute_success_rate(samples,labels,fc_centers)
    print("FriendlyCore success = %lf" % fc_success)   
    fc_list.append(fc_success) 
    
    
       
    lsh_centers = LSH_Clustering(samples,d,k,eps,delta,radius = Lambda)
    lsh_success = compute_success_rate(samples,labels,lsh_centers)
    print("LSH (eps=%lf) success = %lf" % (eps,lsh_success))
    lsh_list.append(lsh_success)

    
if __name__ == '__main__':

    d_options = [100,300,500,700,900]
    num_of_d_options = len(d_options)
    
    fc_suc_lists = []
    lsh_suc_lists = []
    for _ in range(num_of_d_options):
        fc_suc_lists = fc_suc_lists + [[]]
        lsh_suc_lists = lsh_suc_lists + [[]]
        
    fc_suc = []
    fc_stdev = []
    fc_medians = []
    fc_perc90 = []
    fc_perc10 = []
    lsh_suc = []
    lsh_stdev = []
    lsh_medians = []
    lsh_perc90 = []
    lsh_perc10 = []
    counters = []
        
    for i in range(num_of_d_options):
        d = d_options[i]
        print("n = %d, m = %d" % (n,m))
        counters.append(d)
        fc_suc_list = fc_suc_lists[i]
        lsh_suc_list = lsh_suc_lists[i]
        print("Testing d=%d" % d)
        for j in range(iterations):
            print("d=%d, iteration %d" % (d,j))
            single_test(n,d,fc_suc_lists[i],lsh_suc_lists[i])
        print("********   d=%d Summary After Count %d  ********" % (n,i))
        print("FriendlyCore:")
        fc_suc.append(stats.trim_mean(fc_suc_list,0.1)) 
        fc_stdev.append(statistics.pstdev(fc_suc_list))
        fc_medians.append(statistics.median(fc_suc_list))
        fc_perc90.append(np.percentile(fc_suc_list,90))
        fc_perc10.append(np.percentile(fc_suc_list,10))
        print(fc_suc)
        print(fc_stdev)
        print(fc_perc90)
        print(fc_perc10)
        print("LSH:")
        lsh_suc.append(stats.trim_mean(lsh_suc_list,0.1)) 
        lsh_stdev.append(statistics.pstdev(lsh_suc_list))
        lsh_medians.append(statistics.median(lsh_suc_list))
        lsh_perc90.append(np.percentile(lsh_suc_list,90))
        lsh_perc10.append(np.percentile(lsh_suc_list,10))
        print(lsh_suc)
        print(lsh_stdev)
        print(lsh_perc90)
        print(lsh_perc10)
        
        
        np.savetxt("./results/clustering/pca/counters.txt", np.array(counters))
        
        np.savetxt("./results/clustering/pca/fc_centers.txt", np.array(fc_suc))
        np.savetxt("./results/clustering/pca/fc_medians.txt", np.array(fc_medians))
        np.savetxt("./results/clustering/pca/fc_perc90.txt", np.array(fc_perc90))
        np.savetxt("./results/clustering/pca/fc_perc10.txt", np.array(fc_perc10))
        
        np.savetxt("./results/clustering/pca/lsh_centers.txt", np.array(lsh_suc))
        np.savetxt("./results/clustering/pca/lsh_medians.txt", np.array(lsh_medians))
        np.savetxt("./results/clustering/pca/lsh_perc90.txt", np.array(lsh_perc90))
        np.savetxt("./results/clustering/pca/lsh_perc10.txt", np.array(lsh_perc10))    
            
